import logging
from uuid import uuid4, UUID
from dataclasses import dataclass

@dataclass
class LogResult:
    id: UUID
    msg: str

class LoggerWithID(logging.Logger):
    def __init__(self, name, level=NOTSET):
        super().__init__(name, level=level)

    def _log(self, level, msg, args, **kwargs):
        unique_id = uuid4()
        msg = f"{msg} [LOG_ID: {unique_id}]"

        super()._log(level, msg, args, **kwargs)
        
        return LogResult(id=unique_id, msg=msg)



from abc import ABC, abstractmethod
from typing import Any, List, TypeVar, Union

from pydantic import BaseModel

TModel = TypeVar('TModel')

class AbstractAsyncRepository(ABC):
    Model = BaseModel
    OModel = BaseModel

    def __init__(self, session: Any) -> None:
        self.session = session

    @abstractmethod
    async def add(self, obj: Model) -> Union[Model, None]:
        raise NotImplementedError

    @abstractmethod
    async def get(self, *identifiers: List[Any]) -> Union[Model, None]:
        raise NotImplementedError

    async def update(self, obj: Model) -> Union[Model, None]:
        raise NotImplementedError

    async def delete(self, obj: Model) -> None:
        raise NotImplementedError

    async def all(self, *identifier: List[Any]) -> Union[List[OModel], None]:
        raise NotImplementedError
